#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2020 University of Liège
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


# test multiscale - version "task pull"
# linux:
#   mpiexec.openmpi --bycore --bind-to-core -n 6 ./run.py heat/tests/fe2/heat_mpi.py
# windows (msmpi):
#   mpiexec -n 6 python run.py heat\tests\fe2\heat_mpi.py


import heat as h
import heat.utils as u
import tbox
import tbox.gmsh as gmsh
from tbox.solvers import LinearSolver
import heat.fe2 as fe2
from fwk.testing import *

class Micro:
    def __init__(self):
        self.msh = gmsh.MeshLoader("micro.geo", __file__).execute()
        self.gmshWriter = tbox.GmshExport(self.msh)
        self.pbl = h.Problem(self.msh)

        self.pbl.add(h.Medium(self.msh, "Matrix",    tbox.Fct2C(1.,1.,0), 1.0))
        self.pbl.add(h.Medium(self.msh, "Inclusion", tbox.Fct2C(100.,100.,0), 1.0))
        self.p = h.Periodic(self.pbl, 0.0, 0.0, 0.0)
        self.pbl.add(self.p)
        
        self.solver = h.Solver(self.pbl, LinearSolver().pardiso())
        self.solver.nthreads = 1
        self.solver.save = False
        self.solver.verbose = 0
             
class Macro:
    def __init__(self, law):
        self.msh = gmsh.MeshLoader("macro.geo", __file__).execute()
        self.gmshWriter = tbox.GmshExport(self.msh)
        self.pbl = h.Problem(self.msh)

        self.pbl.add(h.Medium(self.msh, "S1", law))
        self.pbl.add(h.Dirichlet(self.msh,"Left", 0.))
        self.pbl.add(h.Dirichlet(self.msh,"Right", 10.))

        self.solver = h.Solver(self.pbl, LinearSolver().pardiso())
        self.solver.nthreads = 1
        self.solver.restol = 1e-6  

if __name__ == "__main__":

    # le gmsh ne doit pas etre fait par toutes les machines
    if fe2.rank==0:
        micro = Micro()
    fe2.barrier() # ok uniqut sur 1node

    if fe2.rank==0:
        law = fe2.FE2(micro)
        macro = Macro(law)
        master = fe2.Master(macro)
        master.start()
    else:
        micro = Micro()
        worker = fe2.Worker(micro)
        worker.start()

    if fe2.rank==0:
        # extract some results
        x1,y1,T1 = u.getTemp(macro.solver,"Bottom", sort='x')
        
        u.simpleplot(x1, T1, xlabel='x [mm]', title='Temperature along x')
           
        tests = CTests()
        tests.add(CTest('mean(Tleft)', sum(T1) / len(T1), 4.683412, 2e-5))
        tests.add(CTest('max(Tleft)', max(T1), 10.0))
        tests.add(CTest('min(Tleft)', min(T1), 0.0))
        tests.run()
   


